<?php

/**
 * PHP飞信发送类
 * @author quanhengzhuang <blog.quanhz.com>
 * @version 1.5.0
 */
class fetion{
    //发送者手机号
    protected $mobile;
    //飞信密码
    protected $password;
    //Cookie字符串
    protected $cookie = '';
    //Uid缓存
    protected $uids = array();
    //csrfToken
    protected $csrfToten = null;
    /**
     * 构造函数
     * @param string $mobile 手机号(登录者)
     * @param string $password 飞信密码
     */
    public function __construct($mobile, $password){
        if ($mobile === '' || $password === ''){
            return;
        }        
        $this->mobile = $mobile;
        $this->password = $password;        
        $this->login();
    }
    //析构函数
    public function __destruct(){
        $this->logout();
    }
    /**
     * 登录
     * @return string
     */
    protected function login()
    {
        $uri = '/huc/user/space/login.do?m=submit&fr=space';
        $data = 'mobilenum='.$this->mobile.'&password='.urlencode($this->password);        
        $result = $this->postWithCookie($uri, $data);
        //解析Cookie
        preg_match_all('/.*?\r\nSet-Cookie: (.*?);.*?/si', $result, $matches);
        if (isset($matches[1])){
            $this->cookie = implode('; ', $matches[1]);
        }        
        $result = $this->postWithCookie('/im/login/cklogin.action', '');
        return $result;
    }
    /**
     * 向指定的手机号发送飞信
     * @param string $mobile 手机号(接收者)
     * @param string $message 短信内容
     * @return string
     */
    public function send($mobile, $message){
        if ($message === ''){
            return '';
        }
        //判断是给自己发还是给好友发
        if ($mobile == $this->mobile){
            return $this->toMyself($message);
        }
        else{
            $uid = $this->getUid($mobile);
            return $uid === '' ? '' : $this->toUid($uid, $message);
        }
    }
    /**
     * 获取飞信ID
     * @param string $mobile 手机号
     * @return string
     */
    protected function getUid($mobile){
        if (empty($this->uids[$mobile])){
            $uri = '/im/index/searchOtherInfoList.action';
            $data = 'searchText='.$mobile;            
            $result = $this->postWithCookie($uri, $data);            
            //匹配
            preg_match('/toinputMsg\.action\?touserid=(\d+)/si', $result, $matches);
            $this->uids[$mobile] = isset($matches[1]) ? $matches[1] : '';
        }        
        return $this->uids[$mobile];
    }
    /**
     * 获取csrfToken，给好友发飞信时需要这个字段
     * @param string $uid 飞信ID
     * @return string
     */
    protected function getCsrfToken($uid){
        if ($this->csrfToten === null){
            $uri = '/im/chat/toinputMsg.action?touserid='.$uid;            
            $result = $this->postWithCookie($uri, '');            
            preg_match('/name="csrfToken".*?value="(.*?)"/', $result, $matches);
            $this->csrfToten = isset($matches[1]) ? $matches[1] : '';
        }
        return $this->csrfToten;
    }
    /**
     * 向好友发送飞信
     * @param string $uid 飞信ID
     * @param string $message 短信内容
     * @return string
     */
    protected function toUid($uid, $message){
        $uri = '/im/chat/sendMsg.action?touserid='.$uid;
        $csrfToken = $this->getCsrfToken($uid);
        $data = 'msg='.urlencode($message).'&csrfToken='.$csrfToken;        
        $result = $this->postWithCookie($uri, $data);        
        return $result;
    }
    /**
     * 给自己发飞信
     * @param string $message
     * @return string
     */
    protected function toMyself($message){
        $uri = '/im/user/sendMsgToMyselfs.action';
        $result = $this->postWithCookie($uri, 'msg='.urlencode($message));
        return $result;
    }
    /**
     * 退出飞信
     * @return string
     */
    protected function logout(){
        $uri = '/im/index/logoutsubmit.action';
        $result = $this->postWithCookie($uri, '');        
        return $result;
    }
    /**
     * 携带Cookie向f.10086.cn发送POST请求
     * @param string $uri
     * @param string $data
     */
    protected function postWithCookie($uri, $data){
        $fp = fsockopen('f.10086.cn', 80);
        fputs($fp, "POST $uri HTTP/1.1\r\n");
        fputs($fp, "Host: f.10086.cn\r\n");
        fputs($fp, "Cookie: {$this->cookie}\r\n");
        fputs($fp, "Content-Type: application/x-www-form-urlencoded\r\n");
        fputs($fp, "User-Agent: Mozilla/5.0 (Windows NT 5.1; rv:14.0) Gecko/20100101 Firefox/14.0.1\r\n");
        fputs($fp, "Content-Length: ".strlen($data)."\r\n");
        fputs($fp, "Connection: close\r\n\r\n");
        fputs($fp, $data);
        $result = '';
        while (!feof($fp)){
            $result .= fgets($fp);
        }
        fclose($fp);
        return $result;
    }
}