<?php
/**
 * 模板操作类
 * 
 * @example
 * 
 * 常量类型标签，必须为大写
 * {DIR} 系统安装目录
 * {T}   模板路径
 * {DIR.T}  等价于{DIR}{T}
 * 
 * 变量类型标签
 * {$title}  普通标签，支持size/code等属性值
 * {$site.name} 带.的是系统内置的参数标签，如网站名称/网站URL：{$site.url}
 *
 * URL参数中获取值的标签
 * {$_GET['title']} 也可以简化成：{$_GET[title]}
 *
 * POST参数中获取值的标签
 * {$_POST['title']}
 *
 * foreach循环标签
 * {#foreach($data as $rs)}
 *		{$rs['title']}
 * {#endforeach}
 * 
 * for循环标签
 * {#for($i=0;$i<10;$i++)}
 *		{$i}
 * {#endfor}
 * 
 * while循环标签
 * {#while($i<10)}
 * {$i++}
 * {#endwhile}
 * 
 * 结束当前循环结构的执行
 * {#break}
 * {#continue}
 * 
 * switch
 * {# switch($var) }
 * {# case 1 }
 *  ...x1...
 * {#break}
 * {# case 2}
 *	...x2...
 * {#break}
 * {#endswitch}
 * 
 * if判断标签
 * {#if(true)}
 * {#elseif(false)}
 * {#else}
 * {#endif}
 * 
 * 文件引用
 * {#include file='include/xxx.php'}
 * 
 * 读取数据库并返回二维数组,返回数组赋给@后面的变量，即data
 * {@data table='%s_user' where='' order=''}
 *
 * 读取数据库并返回一维数组
 * {@data.one table='%s_user' field='*' where='' order=''}
 * 
 * 关联读取数据库，并返回二维数组
 * {@data.join table1='%s_user' table2='%s_content' field1='*' field2='*' on='userid' where='t1.status=1' order="t1.userid desc"}
 * 
 * 关联读取数据库，并返回一维数组
 * {@data.join_one table1='%s_user' table2='%s_content' field1='*' field2='*' on='userid' where='t1.status=1' order="t1.userid desc"}
 *
 * 双字段读取，并返回一维数组
 * {@data.two table='%s_user' field1='userid' field2='username'}
 */

class template {
	
	function path($path){
		
		if(kc_config('switch.site')==0) kc_tip('网站正在维护中……请稍后再来!');
		
		$old_path=ROOT.T.$path;//原始模板文件路径
		//检查模板文件最后一次修改时间
		if(is_file($old_path)){
			$old_time=filemtime(ROOT.T.$path);
		}elseif(is_file(dirname(ROOT.T.$path).'/default.php')){//文件不存在的时候，找找该目录下的default.php文件
			$old_path=dirname(ROOT.T.$path).'/default.php';
			$old_time=filemtime($old_path);
		}elseif(substr($path,-1)=='/' && is_file(ROOT.T.$path.'default.php')){
			$old_path=ROOT.T.$path.'default.php';
			$old_time=filemtime($old_path);
			$path.='default.php';
		}else{
			kc_tip('<p>错误[10001]模板文件不存在! </p><p>&gt;&gt;&gt; '.$path.' &lt;&lt;&lt;</p>');
		}
		
		//记录原始模板的最后修改时间的文件
		$time_path=PATH_CACHE.'/template_time/'.$path;
		$time_time=is_file(ROOT.$time_path) ? file_get_contents(ROOT.$time_path) : 0;
		
		//被缓存的模板文件
		$cache_path=PATH_CACHE.'/template/'.$path;
		//如果两个时间一致，则直接include
		if($time_time==$old_time && is_file(ROOT.$cache_path) && kc_config('switch.template')){
			return $cache_path;
		}		

		//读取原始模板，并解析后存储到缓存里
		$s=file_get_contents($old_path);
	
		//替换系统常量
		$pattern='/\{([A-Z\_]+(\.[A-Z\_]+)*)\}/';
		$replace='<?php echo $1 ?>';
		$s=preg_replace($pattern,$replace,$s);
		
		//精简的<? 改成完整的<?php
		$s=preg_replace('/\<\?\s+/','<?php ',$s);
		/*替换{? ... ?} 为<?php   ?> */
		$s=preg_replace('/\{\?\=((.(?!\{\?\=))*?)\s*\?\}/','<?php echo $1 ?>',$s);
		$s=preg_replace('/\{\?\s*((.(?!\{\?\s))*?)\s*\?\}/is','<?php $1 ?>',$s);
			
		//替换变量、kc_config值、类成员对象
		$pattern='#(<!--\s*)?(\{\$(\w+(\.\w+)?(\-\>\w+)?)(((\[((?>(\'|")?\$\w+)?[^\[\]]+\10?)|(?R))*\])*)((\s+([\w\-]+)=(\'|")(.+?)\14)*)\s*\})(\s*-->)?#is';
		$s=preg_replace_callback($pattern,array(&$this,'regexcallback_var'),$s);
		
		//语法替换,需要补充冒号
		$pattern='/(<!--\s*)?(\{\#)\s*(foreach|for|if|elseif|else|while|switch|default|case)([^}]*)\s*(\})(\s*-->)?/';
		$s=preg_replace_callback($pattern,array(&$this,'regexcallback_statements'),$s);
		
		//语法替换，不需要补充冒号
		$pattern='/(<!--\s*)?(\{\#)\s*(endforeach|endfor|endif|endwhile|break|continue|endswitch)([^}]*)\s*(\})(\s*-->)?/';
		$replace='<?php $3$4 ?>';
		$s=preg_replace($pattern,$replace,$s);
		
		//include文件的处理
		$pattern='#(<!--\s*)?(\{\#)\s*(include|require)\s+file=([\'|"])([^}]+?)\4\s*(\})(\s*-->)?#is';
		$s=preg_replace_callback($pattern,array(&$this,'regexcallback_include'),$s);

		//查询并替换数据库读取部分代码
		$pattern='#(<!--\s*)?\{@([\w\_]+)(\.(one|get|join|one_join|two|select|select_join|count|search|related))?((\s+(\w+)=(["\'])(.*?)\8)*)\s*\}(\s*-->)?#is';
		$s=preg_replace_callback($pattern,array(&$this,'regexcallback_db'),$s);

		//删除模板里的注释,跟普通注释不同的是，多一个-
		$pattern='/\s*(<!-{3,}\s*)[^->]*(\s*-{3,}>)\s*/';
		$replace='';
		$s=preg_replace($pattern,$replace,$s);
		
		$replace="<?php \$vars=get_defined_vars();echo '<div style=\"text-align:left;border:5px solid #CCC;background:#EFEEEE;padding:15px;line-height:20px;\">';";
		$replace.="foreach(\$vars as \$k=>\$r){if(is_string(\$r)){echo '<strong>{\$'.\$k.'}</strong> -&gt; '.\$r.'<br/>';}";
		$replace.="};echo '</div>'; ?>";
		$s=str_replace('{Tags}',$replace,$s);
		
		$file=new file;
		$file->put($cache_path,$s);
		$file->put($time_path,$old_time);
		
		return $cache_path;
	}
	function regexcallback_statements($m){
		if($m[3]=='switch'){			
			return '<?php '.$m[3].$m[4]." : case '@SinCS': ?>";
		}
		return '<?php '.$m[3].$m[4].': ?>';
	}
	
	//变量解析
	function regexcallback_var($m){

		if(!empty($m[5])){//输出config表中值
			$s='$'.$m[3];
			$s.=preg_replace('/\[(\'|")?(\w+)\1?\]/',"['$2']",$m[6]);
		}elseif(!empty($m[4])){//输出config表中值
			$s="kc_config('{$m[3]}')";
		}elseif(!empty($m[6])){//数组形式数据
			$s='$'.$m[3];//.$m[3].$m[5];
			$s.=preg_replace('/\[(\'|")?(\w+)\1?\]/',"['$2']",$m[6]);
		}else{//默认数据类型
			$s='$'.$m[3];
		}
		
		$a=array();
		preg_match_all('/([\w\-]+)=(["\'])(.*?)\2/', $m[11], $mat, PREG_SET_ORDER);
		$isHtmlencode=true;
		foreach($mat as $r){
			$a[strtolower($r[1])]=$r[3];
		}
		foreach($a as $k=>$v){
			switch($k){
				case 'size':
					if(kc_validate($v,2)) $s="\$str->substring({$s},0,{$v})";
					break;
				case 'code':
					$v=strtolower($v);
					if($v=='html'||$v=='htmldecode'){
						$isHtmlencode=false;
					}elseif($v=='quote'){
						$s="\$str->quoteEncode($s)";
					}elseif($v=='url'||$v=='urlencode'){
						$s="urlencode($s)";
					}elseif($v=='nl2br'){
						$s="nl2br(htmlspecialchars($s))";
						$isHtmlencode=false;
					}elseif($v=='ip'){
						$s="\$str->formatIp(long2ip($s))";
					}elseif($v=='json'){
						$s="json_encode($s)";
						$isHtmlencode=false;
					}elseif($v=='long2ip'){
						$s="long2ip($s)";
					}elseif($v=='price'){
						$s="\$str->price_format($s)";
					}elseif($v=='order_status'){
						$s="order_status($s)";
					}
					break;
				case 'date':
					$s="\$str->formatDate({$s},'{$v}')";
					break;
			}
		}
		//图片处理部分，只生成图片路径并加上sign，以免被恶意创建缩略图
		if(isset($a['width'])||isset($a['height'])){
			$a['width']=empty($a['width']) ? 120 : $a['width'];
			$a['height']=empty($a['height']) ? 90 : $a['height'];
			$a['auto']=empty($a['auto']) ? '' : $a['auto'];
			$array_image=array('width','height','background','position','auto','padding','padding-color','padding-size','padding-r','padding-filter','border','border-color','border-margin','border-r','border-size');
			$array_url=array();
			foreach($array_image as $v){
				if(!empty($a[$v])){
					$array_url[$v]=$a[$v];
				}
			}
			ksort($array_url);
			$param=http_build_query($array_url);
			return "<?='images/?id='.$s.'&$param&sign='.md5('id='.$s.'$param'.kc_config('system.salt')).'.jpg'?>";
		}
		$s=$isHtmlencode?"htmlspecialchars($s)":$s;
		$s="<?={$s}?>";
		return $s;
	}
	//给图片路径做加密
	function sign_image($id,$attrib){
		ksort($attrib);
		$param=  http_build_query($attrib);
		return DIR."images/?id={$id}&{$param}&sign=".md5('id='.$id.$param.kc_config('system.salt')).'.jpg';
	}
	//数据库标签的解析
	function regexcallback_db($m){
		$cache_path='$cache_path';
		preg_match_all('/(\w+)=(["\'])(.*?)\2/', $m[5], $mat, PREG_SET_ORDER);
		$a=array();
		$salt=$m[4];
		foreach($mat as $r){
			$r[3]=preg_replace('/\<\?\=([A-Z\_]+(\.[A-Z\_]+)*)\?\>/','".$1."',$r[3]);
			$a[$r[1]]=preg_replace('/(\<\?\=htmlspecialchars\()(\$.+?)(\)\?\>)/','{$2}',$r[3]);
			$salt.=';'.$r[1].'='.$a[$r[1]];
		}		
		
		$where=empty($a['where'])?'':$a['where'];
		$group=empty($a['group'])?'':$a['group'];
		$order=empty($a['order'])?'':$a['order'];
		$cache=isset($a['cache'])?$a['cache']:600;
		if(!preg_match('/^\d+$/',$cache)) $cache=600;
		$rn=empty($a['number'])?'10':$a['number'];
		switch ($m[4]) {
		case '':
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			if(empty($a['table'])) return $m[0];
			$t1=$a['table'];
			$f1=empty($a['field'])?'*':$a['field'];
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->getRows(\"$t1\",\"$f1\",\"$where\",\"$order\",\"$rn\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'get':
			if(empty($a['sql'])) return $m[0];
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->get(\"{$a['sql']}\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'one':
			if(empty($a['table'])) return $m[0];
			$t1=$a['table'];
			$f1=empty($a['field'])?'*':$a['field'];
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->getRows_one(\"$t1\",\"$f1\",\"$where\",\"$order\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'two':
			if(empty($a['table'])||empty($a['field1'])||empty($a['field2'])) return $m[0];
			$t1=$a['table'];
			$f1=$a['field1'];
			$f2=$a['field2'];
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->getRows_two(\"$t1\",\"$f1\",\"$f2\",\"$where\",\"$order\",\"$rn\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'count':
			if(empty($a['table'])) return $m[0];
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->getCount(\"{$a['table']}\",\"{$where}\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'join':
			if(empty($a['table1'])||empty($a['table2'])||empty($a['on'])) return $m[0];
			$i=1;$tables=array();
			if(!empty($a['field'])) $tables[]=array('field'=>$a['field']);
			while(!empty($a['table'.$i])){
				$tables[]=array(
					'table'=>$a['table'.$i],
					'field'=>empty($a['field'.$i])?'':$a['field'.$i]
				);
				$i++;
			}
			$on=$a['on'];
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->getRows_more(".var_export($tables,1).",\"$on\",\"$where\",\"$order\",\"$rn\",\"$group\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'one_join':
			if(empty($a['table1'])||empty($a['table2'])||empty($a['on'])) return $m[0];
			$i=1;$tables=array();
			if(!empty($a['field'])) $tables[]=array('field'=>$a['field']);
			while(!empty($a['table'.$i])){
				$tables[]=array(
					'table'=>$a['table'.$i],
					'field'=>empty($a['field'.$i])?'':$a['field'.$i]
				);
				$i++;
			}
			$on=$a['on'];
			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\${$m[2]}=\$cache->get(\$cache_path);\nif(empty(\${$m[2]})){\n";
			$s.="\${$m[2]}=\$db->getRows_one_more(".var_export($tables,1).",\"$on\",\"$where\",\"$order\");\n";
			$s.="\$cache->set(\$cache_path,\${$m[2]},$cache);}\n";
			$s.=" ?>";
			break;
		case 'select':
			if(empty($a['table'])) return $m[0];
			$t1=$a['table'];
			$f1=empty($a['field'])?'*':$a['field'];
			$url=empty($a['url'])?DIR.PATH:$a['url'];
			$inner=empty($a['pagelist']) ?  '<p class="kc_page">[standard][next]</p>' :$a['pagelist'];

			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\".PID);";
			$s.="\$cache_array_temp=\$cache->get(\$cache_path);\nif(empty(\$cache_array_temp)){\n";
			$s.="\${$m[2]}=\$db->select(\"$t1\",\"$f1\",\"$url\",\"$where\",\"$order\",PID,\"$rn\",'$inner');\n";
			$s.="\${$m[2]}_pagelist=\$db->pagelist;\n";
			$s.="\$cache_array_temp=array('data'=>\${$m[2]},'pagelist'=>\$db->pagelist);\n";
			$s.="\$cache->set(\$cache_path,\$cache_array_temp,$cache);}\n";
			$s.="else{\${$m[2]}=\$cache_array_temp['data'];\${$m[2]}_pagelist=\$cache_array_temp['pagelist'];}\n";
			$s.=" ?>";
			break;
		case 'select_join':
			if(empty($a['table1'])||empty($a['table2'])||empty($a['on'])) return $m[0];
			$i=1;$tables=array();
			if(!empty($a['field'])) $tables[]=array('field'=>$a['field']);
			while(!empty($a['table'.$i])){
				$tables[]=array(
					'table'=>$a['table'.$i],
					'field'=>empty($a['field'.$i])?'':$a['field'.$i]
				);
				$i++;
			}
			$on=$a['on'];
			$url=empty($a['url'])?DIR.PATH:$a['url'];
			$inner=empty($a['pagelist']) ?  '<p class="kc_page">[standard][next]</p>' :$a['pagelist'];

			$s="<?php \$cache_path='template_db_block/'.md5(\"$salt\".PID);";
			$s.="\$cache_array_temp=\$cache->get(\$cache_path);\nif(empty(\$cache_array_temp)){\n";
			$s.="\${$m[2]}=\$db->select_more(".var_export($tables,1).",\"$on\",\"$url\",\"$where\",\"$order\",PID,\"$rn\",'$inner');\n";
			$s.="\${$m[2]}_pagelist=\$db->pagelist;\n";
			$s.="\$cache_array_temp=array('data'=>\${$m[2]},'pagelist'=>\$db->pagelist);\n";
			$s.="\$cache->set(\$cache_path,\$cache_array_temp,$cache);\n}";
			$s.="else{\${$m[2]}=\$cache_array_temp['data'];\${$m[2]}_pagelist=\$cache_array_temp['pagelist'];}\n";
			$s.=" ?>";
			break;
		case 'search':
			if(empty($a['table'])||empty($a['fulltext'])||empty($a['field'])) return $m[0];
			if(empty($_GET['q'])) return "<?php \${$m[2]}=array();\${$m[2]}_pagelist=''; ?>";

			$t=str_replace('%s',DB_PRE,$a['table']);
			$ft=$a['fulltext'];
			$field=$a['field'];
			$sql="SELECT $field,MATCH(`$ft`) AGAINST('\$ws' IN BOOLEAN MODE) AS temp_score";
			$sql.=" FROM $t WHERE MATCH(`$ft`) AGAINST('\$ws' IN BOOLEAN MODE)";
			$sql.=empty($where)?'':' AND '.$where;
			$sql.=empty($order)?' ORDER BY temp_score desc':' ORDER BY '.$order;
			$inner=empty($a['pagelist']) ?  '<p class="kc_page">[standard][next]</p>' :$a['pagelist'];
			$sql.=" LIMIT \".($rn*(PID-1)).\",$rn ";

			$s="<?php \$key=new keywords;\$ws=\$key->wordSegment(\$_GET['q']);";
			$s.="\$cache_path='template_db_block/'.md5(\"$salt\".\$_GET['q'].PID);";
			$s.="\$cache_array_temp=\$cache->get(\$cache_path);\nif(empty(\$cache_array_temp)){\n";
			global $Q;
			$s.="\${$m[2]}=\$db->get(\"$sql\");\n";
			$s.="\$rcount=\$db->getCount(\"{$t}\",\"MATCH(`$ft`) AGAINST('\$ws' IN BOOLEAN MODE)".(empty($where)?'':' AND '.$where)."\");\n";
			$s.="\${$m[2]}_pagelist=\$str->pagelist('?".(empty($Q['ismobile'])?'':'m&')."q='.urlencode(\$_GET['q']).'&app='.urlencode(empty(\$_GET['app'])?\$app:\$_GET['app']).'&pid=PID',\$rcount,PID,$rn,'$inner');\n";
			$s.="\$cache_array_temp=array('data'=>\${$m[2]},'pagelist'=>\${$m[2]}_pagelist);\n";
			$s.="\$cache->set(\$cache_path,\$cache_array_temp,$cache);}\n";
			$s.="else{\${$m[2]}=\$cache_array_temp['data'];\${$m[2]}_pagelist=\$cache_array_temp['pagelist'];}\n";
			$s.=" ?>";
			break;
		case 'related':

			if(empty($a['table'])||empty($a['fulltext'])||empty($a['field'])||empty($a['word'])) return $m[0];

			$t=str_replace('%s',DB_PRE,$a['table']);
			$ft=$a['fulltext'];
			$field=$a['field'];
			$ws=$a['word'];
			$sql="SELECT $field,MATCH(`$ft`) AGAINST('$ws' IN BOOLEAN MODE) AS temp_score";
			$sql.=" FROM $t WHERE MATCH(`$ft`) AGAINST('$ws' IN BOOLEAN MODE)";
			$sql.=empty($where)?'':' AND '.$where;
			$sql.=empty($order)?' ORDER BY temp_score desc':' ORDER BY '.$order;
			$sql.=" LIMIT $rn ";

			$s="<?php ";
			$s.="\$cache_path='template_db_block/'.md5(\"$salt\");";
			$s.="\$cache_array_temp=\$cache->get(\$cache_path);\nif(empty(\$cache_array_temp)){\n";
			$s.="\${$m[2]}=\$db->get(\"$sql\");\n";
			$s.="\$cache_array_temp=array('data'=>\${$m[2]});\n";
			$s.="\$cache->set(\$cache_path,\$cache_array_temp,$cache);}\n";
			$s.="else{\${$m[2]}=\$cache_array_temp['data'];}\n";
			$s.=" ?>";
			break;
		}
		return $s;
	}
	
	function regexcallback_include($m){
		$t=new template;
		$cache_path=$t->path($m[5]);
		return "<?php include ROOT.'{$cache_path}';?>";
	}	
}
?>